@@ -1,9 +1,9 @@
 /*
- * $Id: dialog.c,v 1.291 2024/01/01 10:10:37 tom Exp $
+ * $Id: dialog.c,v 1.296 2025/08/17 20:04:40 tom Exp $
  *
  *  cdialog - Display simple dialog boxes from shell scripts
  *
- *  Copyright 2000-2023,2024	Thomas E. Dickey
+ *  Copyright 2000-2024,2025	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -41,12 +41,13 @@ typedef enum {
     ,o_aspect_ratio
     ,o_auto_placement
     ,o_backtitle
-    ,o_beep_signal
     ,o_beep_after_signal
+    ,o_beep_signal
     ,o_begin_set
     ,o_cancel_label
+    ,o_cant_kill
     ,o_checklist
-    ,o_dlg_clear_screen
+    ,o_color_mode
     ,o_colors
     ,o_column_separator
     ,o_cr_wrap
@@ -56,6 +57,7 @@ typedef enum {
     ,o_default_button
     ,o_default_item
     ,o_defaultno
+    ,o_dlg_clear_screen
     ,o_erase_on_exit
     ,o_exit_label
     ,o_extra_button
@@ -88,27 +90,26 @@ typedef enum {
     ,o_mixedgauge
     ,o_msgbox
     ,o_no_close
-    ,o_nocollapse
     ,o_no_cr_wrap
-    ,o_cant_kill
     ,o_no_hot_list
     ,o_no_label
     ,o_no_lines
     ,o_no_mouse
     ,o_no_nl_expand
-    ,o_use_shadow
     ,o_nocancel
+    ,o_nocollapse
     ,o_nook
     ,o_ok_label
     ,o_output_fd
     ,o_output_separator
+    ,o_output_stderr
+    ,o_output_stdout
     ,o_passwordbox
     ,o_passwordform
     ,o_pause
     ,o_prgbox
     ,o_print_maxsize
     ,o_print_siz
-    ,o_text_only
     ,o_print_text_size
     ,o_print_version
     ,o_programbox
@@ -116,25 +117,25 @@ typedef enum {
     ,o_quoted
     ,o_radiolist
     ,o_screen_center
-    ,o_use_scrollbar
     ,o_separate_output
     ,o_separate_str
     ,o_single_quoted
     ,o_size_err
     ,o_sleep_secs
     ,o_smooth
-    ,o_output_stderr
-    ,o_output_stdout
     ,o_tab_correct
     ,o_tab_len
     ,o_tailbox
     ,o_tailboxbg
+    ,o_text_only
     ,o_textbox
     ,o_time_format
     ,o_timeout_secs
     ,o_title
     ,o_trim_whitespace
     ,o_under_mouse
+    ,o_use_scrollbar
+    ,o_use_shadow
     ,o_version
     ,o_visit_items
     ,o_wmclass
@@ -227,7 +228,7 @@ static int known_opts = 0;
 static const char **dialog_opts;
 static char **dialog_argv;
 
-static char **special_argv = 0;
+static char **special_argv = NULL;
 static int special_argc = 0;
 
 static bool ignore_unknown = FALSE;
@@ -266,6 +267,7 @@ static const Options options[] = {
     { "cancel-label",	svS(cancel_label),	1, "<str>" },
     { "checklist",	opW(checklist),		2, "<text> <height> <width> <list height> <tag1> <item1> <status1>..." },
     { "clear",		svT(dlg_clear_screen),	1, "" },
+    { "color-mode",	ssN(color_mode),	1, "<mask>" },
     { "colors",		svT(colors),		1, "" },
     { "column-separator",svS(column_separator),	1, "<str>" },
     { "cr-wrap",	svT(cr_wrap),		1, "" },
@@ -446,10 +448,10 @@ isOption(const char *arg)
 {
     bool result = FALSE;
 
-    if (arg != 0) {
-	if (dialog_opts != 0) {
+    if (arg != NULL) {
+	if (dialog_opts != NULL) {
 	    int n;
-	    for (n = 0; dialog_opts[n] != 0; ++n) {
+	    for (n = 0; dialog_opts[n] != NULL; ++n) {
 		if (dialog_opts[n] == arg) {
 		    result = TRUE;
 		    break;
@@ -488,15 +490,15 @@ unescape_argv(int *argcp, char ***argvp)
     int count_includes = 0;
     bool doalloc = FALSE;
     char *filename;
-    const char **my_argv = 0;
+    const char **my_argv = NULL;
     int my_argc;
 
     DLG_TRACE(("# unescape_argv\n"));
     for (k = 0; k < 2; ++k) {
 
 	my_argc = 0;
-	if (special_argv != 0) {
-	    for (j = 0; special_argv[j] != 0; ++j) {
+	if (special_argv != NULL) {
+	    for (j = 0; special_argv[j] != NULL; ++j) {
 		if (!strcmp(special_argv[j], "--")) {
 		    break;
 		} else if (isOption(special_argv[j])) {
@@ -532,7 +534,7 @@ unescape_argv(int *argcp, char ***argvp)
 		dlg_exiterr(_("Too many --file options"));
 	    }
 
-	    if ((filename = (*argvp)[j + 1]) != 0) {
+	    if ((filename = (*argvp)[j + 1]) != NULL) {
 		FILE *fp;
 		char **list;
 
@@ -613,7 +615,7 @@ unescape_argv(int *argcp, char ***argvp)
 			(*argvp)[n + j] = list[n];
 		    }
 		    *argcp += added - 2;
-		    (*argvp)[*argcp] = 0;	/* Write the NULL terminator */
+		    (*argvp)[*argcp] = NULL;	/* Write the NULL terminator */
 		    free(list);	/* No-op if 'list' is NULL */
 		    /* Force rescan starting from the first inserted argument */
 		    --j;
@@ -629,7 +631,7 @@ unescape_argv(int *argcp, char ***argvp)
 	    }
 	}
 	if (!escaped
-	    && (*argvp)[j] != 0
+	    && (*argvp)[j] != NULL
 	    && !strncmp((*argvp)[j], "--", (size_t) 2)
 	    && isalpha(UCH((*argvp)[j][2]))) {
 	    my_argv[my_argc++] = (*argvp)[j];
@@ -637,7 +639,7 @@ unescape_argv(int *argcp, char ***argvp)
 	}
     }
 
-    my_argv[my_argc] = 0;
+    my_argv[my_argc] = NULL;
 
     known_opts = my_argc;
     dialog_opts = my_argv;
@@ -712,7 +714,7 @@ arg_rest(char *argv[])
 {
     int i = 1;			/* argv[0] points to a "--" token */
 
-    while (argv[i] != 0
+    while (argv[i] != NULL
 	   && (!isOption(argv[i]) || lookupOption(argv[i], 7) == o_unknown))
 	i++;
     return i;
@@ -728,7 +730,7 @@ howmany_tags(char *argv[], int group)
     int result = 0;
     char temp[80];
 
-    while (argv[0] != 0) {
+    while (argv[0] != NULL) {
 	int have;
 
 	if (isOption(argv[0]))
@@ -756,10 +758,10 @@ numeric_arg(char **av, int n)
     int result = 0;
 
     if (n < dlg_count_argv(av)) {
-	char *last = 0;
+	char *last = NULL;
 	result = (int) strtol(av[n], &last, 10);
 
-	if (last == 0 || *last != 0) {
+	if (last == NULL || *last != 0) {
 	    char msg[80];
 
 	    sprintf(msg, _("Expected a number for token %d of %.*s"), n,
@@ -812,7 +814,7 @@ show_result(int ret)
 		  dialog_state.output);
 	    either = TRUE;
 	}
-	if (dialog_vars.input_result != 0
+	if (dialog_vars.input_result != NULL
 	    && dialog_vars.input_result[0] != '\0') {
 	    fputs(dialog_vars.input_result, dialog_state.output);
 	    DLG_TRACE(("# input_result:\n%s\n", dialog_vars.input_result));
@@ -893,7 +895,7 @@ call_inputmenu(CALLARGS)
 
     dialog_vars.input_menu = TRUE;
 
-    if (dialog_vars.extra_label == 0) {
+    if (dialog_vars.extra_label == NULL) {
 	free_extra_label = TRUE;
 	dialog_vars.extra_label = dlg_strclone(_("Rename"));
     }
@@ -909,7 +911,7 @@ call_inputmenu(CALLARGS)
 			 tags, av + 5);
     if (free_extra_label) {
 	free(dialog_vars.extra_label);
-	dialog_vars.extra_label = 0;
+	dialog_vars.extra_label = NULL;
     }
     return result;
 }
@@ -951,7 +953,7 @@ call_inputbox(CALLARGS)
 			   av[1],
 			   numeric_arg(av, 2),
 			   numeric_arg(av, 3),
-			   optional_str(av, 4, 0), 0);
+			   optional_str(av, 4, NULL), 0);
 }
 
 static int
@@ -962,7 +964,7 @@ call_passwordbox(CALLARGS)
 			   av[1],
 			   numeric_arg(av, 2),
 			   numeric_arg(av, 3),
-			   optional_str(av, 4, 0), 1);
+			   optional_str(av, 4, NULL), 1);
 }
 
 #ifdef HAVE_XDIALOG
@@ -1028,13 +1030,10 @@ call_timebox(CALLARGS)
 
 #define DisableNoTags() \
 	bool save_no_tags = dialog_vars.no_tags; \
-	bool save_no_items = dialog_vars.no_items; \
-	dialog_vars.no_tags = TRUE; \
-	dialog_vars.no_items = FALSE
+	dialog_vars.no_tags = TRUE
 
 #define RestoreNoTags() \
-	dialog_vars.no_tags = save_no_tags; \
-	dialog_vars.no_items = save_no_items
+	dialog_vars.no_tags = save_no_tags
 
 static int
 call_buildlist(CALLARGS)
@@ -1223,7 +1222,7 @@ call_programbox(CALLARGS)
 				numeric_arg(av, 2),
 				TRUE,
 				dialog_state.pipe_input));
-    dialog_state.pipe_input = 0;
+    dialog_state.pipe_input = NULL;
     return result;
 }
 #endif
@@ -1325,7 +1324,7 @@ optionString(char **argv, int *num)
 {
     int next = *num + 1;
     char *result = argv[next];
-    if (result == 0) {
+    if (result == NULL) {
 	char temp[80];
 	sprintf(temp, ("Expected a string-parameter for %.*s"), USEMAX_ARGS,
 		argv[*num]);
@@ -1339,17 +1338,17 @@ static int
 optionValue(char **argv, int *num)
 {
     int next = *num + 1;
-    char *src = argv[next];
-    char *tmp = 0;
+    const char *src = argv[next];
+    char *tmp = NULL;
     int result = 0;
 
-    if (src != 0) {
+    if (src != NULL) {
 	result = (int) strtol(src, &tmp, 0);
-	if (tmp == 0 || *tmp != 0)
-	    src = 0;
+	if (tmp == NULL || *tmp != 0)
+	    src = NULL;
     }
 
-    if (src == 0) {
+    if (src == NULL) {
 	char temp[80];
 	sprintf(temp, _("Expected a numeric-parameter for %.*s"),
 		USEMAX_ARGS, argv[*num]);
@@ -1422,7 +1421,7 @@ button_code(const char *name)
 static void
 IgnoreNonScreen(char **argv, int offset)
 {
-    if (argv[offset + 1] == 0) {
+    if (argv[offset + 1] == NULL) {
 	ignore_unknown = TRUE;
 	end_dialog();
     }
@@ -1484,12 +1483,12 @@ PrintList(const char *const *list)
     const char *leaf = strrchr(program, '/');
     unsigned n = 0;
 
-    if (leaf != 0)
+    if (leaf != NULL)
 	leaf++;
     else
 	leaf = program;
 
-    while (*list != 0) {
+    while (*list != NULL) {
 	fprintf(dialog_state.output, *list, n ? leaf : dialog_version());
 	(void) fputc('\n', dialog_state.output);
 	n = 1;
@@ -1500,7 +1499,7 @@ PrintList(const char *const *list)
 static const Mode *
 lookupMode(eOptions code)
 {
-    const Mode *modePtr = 0;
+    const Mode *modePtr = NULL;
     unsigned n;
 
     for (n = 0; n < TableSize(modes); n++) {
@@ -1538,7 +1537,7 @@ Help(void)
     static const char *const tbl_1[] =
     {
 	"cdialog (ComeOn Dialog!) version %s",
-	"Copyright 2000-2023,2024 Thomas E. Dickey",
+	"Copyright 2000-2024,2025 Thomas E. Dickey",
 	"This is free software; see the source for copying conditions.  There is NO",
 	"warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.",
 	"",
@@ -1551,13 +1550,13 @@ Help(void)
 	"Special options:",
 	"  [--create-rc \"file\"]",
 #endif
-	0
+	NULL
     }, *const tbl_3[] =
     {
 	"",
 	"Auto-size with height and width = 0. Maximize with height and width = -1.",
 	"Global-auto-size if also menu_height/list_height = 0.",
-	0
+	NULL
     };
     size_t limit = TableSize(options);
     size_t j, k;
@@ -1577,7 +1576,7 @@ Help(void)
     fprintf(dialog_state.output, "Common options:\n ");
     for (j = k = 0; j < limit; j++) {
 	if ((opts[j]->pass & 1)
-	    && opts[j]->help != 0) {
+	    && opts[j]->help != NULL) {
 	    size_t len = 6 + strlen(opts[j]->name) + strlen(opts[j]->help);
 	    k += len;
 	    if (k > 75) {
@@ -1591,7 +1590,7 @@ Help(void)
     fprintf(dialog_state.output, "\nBox options:\n");
     for (j = 0; j < limit; j++) {
 	if ((opts[j]->pass & 2) != 0
-	    && opts[j]->help != 0
+	    && opts[j]->help != NULL
 	    && lookupMode(opts[j]->code)) {
 	    fprintf(dialog_state.output, "  --%-12s %s\n", opts[j]->name,
 		    opts[j]->help);
@@ -1616,7 +1615,7 @@ process_trace_option(char **argv, int *offset)
 {
     int j;
 
-    if (dialog_state.trace_output == 0) {
+    if (dialog_state.trace_output == NULL) {
 	dlg_trace(optionString(argv, offset));
     } else {
 	DLG_TRACE(("# ignore extra --trace option\n"));
@@ -1624,7 +1623,7 @@ process_trace_option(char **argv, int *offset)
     }
 
     DLG_TRACE(("# Parameters:\n"));
-    for (j = 0; argv[j] != 0; ++j) {
+    for (j = 0; argv[j] != NULL; ++j) {
 	DLG_TRACE(("#\targv[%d] = %s\n", j, argv[j]));
     }
 }
@@ -1771,7 +1770,7 @@ process_common_options(int argc, char **argv, int offset, int output)
 	    break;
 #endif
 	case o_iso_week:
-	    if (dialog_vars.week_start == 0) {	/* Monday is implied */
+	    if (dialog_vars.week_start == NULL) {	/* Monday is implied */
 		static char default_1st[] = "1";
 		dialog_vars.week_start = default_1st;
 	    }
@@ -1812,16 +1811,16 @@ init_result(char *buffer)
      */
     if (first) {
 	char *env = dlg_getenv_str("DIALOGOPTS");
-	if (env != 0)
+	if (env != NULL)
 	    env = dlg_strclone(env);
-	if (env != 0) {
+	if (env != NULL) {
 	    special_argv = dlg_string_to_argv(env);
 	    special_argc = dlg_count_argv(special_argv);
 	}
 	first = FALSE;
     }
 
-    if (special_argv != 0) {
+    if (special_argv != NULL) {
 	process_common_options(special_argc, special_argv, 0, FALSE);
     }
 }
@@ -1878,14 +1877,14 @@ main(int argc, char *argv[])
 	    break;
 	case o_input_fd:
 	    if ((j = optionValue(argv, &offset)) < 0
-		|| (dialog_state.input = fdopen(j, "r")) == 0) {
+		|| (dialog_state.input = fdopen(j, "r")) == NULL) {
 		handle_leaks();
 		dlg_exiterr(_("Cannot open input-fd\n"));
 	    }
 	    break;
 	case o_output_fd:
 	    if ((j = optionValue(argv, &offset)) < 0
-		|| (dialog_state.output = fdopen(j, "w")) == 0) {
+		|| (dialog_state.output = fdopen(j, "w")) == NULL) {
 		handle_leaks();
 		dlg_exiterr(_("Cannot open output-fd\n"));
 	    }
@@ -2029,10 +2028,10 @@ main(int argc, char *argv[])
 
 	/* use a table to look for the requested mode, to avoid code duplication */
 
-	modePtr = 0;
+	modePtr = NULL;
 	if ((code = lookupOption(argv[offset], 2)) != o_unknown)
 	    modePtr = lookupMode(code);
-	if (modePtr == 0) {
+	if (modePtr == NULL) {
 	    sprintf(temp,
 		    (lookupOption(argv[offset], 7) != o_unknown
 		     ? _("Unexpected option %.*s")
@@ -2074,9 +2073,9 @@ main(int argc, char *argv[])
 	    case o_help_file:
 		break;
 	    default:
-		if (argv[j] != 0) {
+		if (argv[j] != NULL) {
 		    char *argv_j = strdup(argv[j]);
-		    if (argv_j != 0) {
+		    if (argv_j != NULL) {
 			dlg_trim_string(argv_j);
 			argv[j] = argv_j;
 		    } else {
@@ -2097,7 +2096,7 @@ main(int argc, char *argv[])
 
 	if (dialog_vars.input_result != my_buffer) {
 	    free(dialog_vars.input_result);
-	    dialog_vars.input_result = 0;
+	    dialog_vars.input_result = NULL;
 	}
 
 	if (retval == DLG_EXIT_ESC) {
